% A recipe for statistical arbitrage
%
% Copyright 2015 Ali N. Akansu and Mustafa U. Torun
%
% A Primer for Financial Engineering
% Financial Signal Processing and Electronic Trading
%

clear all; close all; clc;

% Parameters
T1 = 1.0; % Threshold for enter
T2 = 0.5; % Threshold for exit
W = 60; % Estimation window length
K = 2; % Number of eigenportfolios

% Load data and calculate return processes
load Data
price = [Adjusted_Price_AAPL ...
    Adjusted_Price_IBM ...
    Adjusted_Price_INTC ...
    Adjusted_Price_MSFT];
N = size(price,1); % number of samples
Ns = size(price,2); % number of stocks

% Calculate eigenporfolios for each estimation window
F = zeros(N,Ns);
for n = W:N
    index = n-W+1:n;   
    R = get_return(price(index,:));    
    Rn = normalize_return(R);      
    C = Rn' * Rn / (length(R) - 1);   
    [PHI,LAMBDA] = eig(C);   
    PHI = fliplr(PHI);
    LAMBDA = rot90(LAMBDA,2);    
    sigma = std(R);
    for k = 1:Ns
        F(n,k) = sum(PHI(:,k) ./ sigma' .* R(end,:)');
    end
end

% Generate the signal and the spread for each stock
R = [zeros(1,Ns); (price(2:end,:) ./ price(1:end-1,:)) - 1];
RN = normalize_return(R);
spread = zeros(N, Ns);
signal = zeros(N, Ns);
pos = NO_POSITION * ones(N, Ns);
for stock = 1:Ns;
    for n = (W * 2 - 1):N
        index = n-W+1:n;
        
        % Regress return on eigenportfolios
        [beta,beta_int,epsilon] = regress(R(index,stock), ...
            [ones(W,1) F(index,1:K)]);
        
        % Calculate the spread
        X = cumsum(epsilon);
        spread(n,stock) = (X(end) - mean(X)) / std(X);
        
        % Once we have the spread, signal generation is similar to 
        % pairs trading
        if pos(n-1,stock) == NO_POSITION && spread(n,stock) > T1
            signal(n,stock) = SHORT_SELL;
            pos(n,stock) = SHORT;
        elseif pos(n-1,stock) == NO_POSITION && spread(n,stock) < -T1
            signal(n,stock) = BUY;
            pos(n,stock) = LONG;
        elseif pos(n-1,stock) == SHORT && spread(n,stock) < T2
            signal(n,stock) = BUY_TO_COVER;
            pos(n,stock) = NO_POSITION;
        elseif pos(n-1,stock) == LONG && spread(n,stock) > -T2
            signal(n,stock) = SELL;
            pos(n,stock) = NO_POSITION;
        else
            pos(n,stock) = pos(n-1,stock);
        end        
    end
end

% Plot spread and position for each stock
for stock = 1:Ns
    figure
    subplot 211
    plot(datenum(Dates), spread(:,stock));
    hold on
    plot(datenum(Dates),  T1 * ones(N,1), 'r');
    plot(datenum(Dates),  T2 * ones(N,1), 'k');
    plot(datenum(Dates), -T2 * ones(N,1), 'k');
    plot(datenum(Dates), -T1 * ones(N,1), 'r');
    datetick('x')
    ylabel('Spread')
    
    subplot 212
    stairs(datenum(Dates), pos(:,stock));
    datetick('x')
    ylabel('Position')
end
